@@ -1,7 +1,10 @@
 /*
  * Copyright (C) 2006, 2007, 2008 OpenedHand Ltd.
+ * Copyright (C) 2009 Nokia Corporation, all rights reserved.
  *
  * Author: Jorn Baayen <jorn@openedhand.com>
+ *         Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
+ *                               <zeeshan.ali@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -42,9 +45,6 @@
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <arpa/inet.h>
-#include <net/if.h>
-#include <ifaddrs.h>
 #include <libsoup/soup-address.h>
 #include <glib/gstdio.h>
 
@@ -59,7 +59,6 @@ G_DEFINE_TYPE (GUPnPContext,
                GSSDP_TYPE_CLIENT);
 
 struct _GUPnPContextPrivate {
-        char        *host_ip;
         guint        port;
 
         guint        subscription_timeout;
@@ -72,15 +71,12 @@ struct _GUPnPContextPrivate {
 
 enum {
         PROP_0,
-        PROP_HOST_IP,
         PROP_PORT,
         PROP_SERVER,
         PROP_SESSION,
         PROP_SUBSCRIPTION_TIMEOUT
 };
 
-#define LOOPBACK_IP "127.0.0.1"
-
 typedef struct {
         char *local_path;
         char *server_path;
@@ -98,153 +94,10 @@ make_server_id (void)
 
         return g_strdup_printf ("%s/%s UPnP/1.0 GUPnP/%s",
                                 sysinfo.sysname,
-                                sysinfo.version,
+                                sysinfo.release,
                                 VERSION);
 }
 
-/*
- * Get the host IP for the specified interface, or the first up and non-loopback
- * interface if no name is specified.
- */
-static char *
-get_host_ip (const char *name)
-{
-        struct ifaddrs *ifa_list, *ifa;
-        char *ret;
-
-        ret = NULL;
-
-        if (getifaddrs (&ifa_list) != 0) {
-                g_error ("Failed to retrieve list of network interfaces:\n%s\n",
-                         strerror (errno));
-
-                return NULL;
-        }
-
-        for (ifa = ifa_list; ifa != NULL; ifa = ifa->ifa_next) {
-                char ip[INET6_ADDRSTRLEN];
-                const char *p;
-                struct sockaddr_in *s4;
-                struct sockaddr_in6 *s6;
-
-                if (ifa->ifa_addr == NULL)
-                        continue;
-
-                if ((ifa->ifa_flags & IFF_LOOPBACK) ||
-                    !(ifa->ifa_flags & IFF_UP))
-                        continue;
-
-                /* If a name was specified, check it */
-                if (name && strcmp (name, ifa->ifa_name) != 0)
-                        continue;
-                
-                p = NULL;
-
-                switch (ifa->ifa_addr->sa_family) {
-                case AF_INET:
-                        s4 = (struct sockaddr_in *) ifa->ifa_addr;
-                        p = inet_ntop (AF_INET,
-                                       &s4->sin_addr, ip, sizeof (ip));
-                        break;
-                case AF_INET6:
-                        s6 = (struct sockaddr_in6 *) ifa->ifa_addr;
-                        p = inet_ntop (AF_INET6,
-                                       &s6->sin6_addr, ip, sizeof (ip));
-                        break;
-                default:
-                        continue; /* Unknown: ignore */
-                }
-
-                if (p != NULL) {
-                        ret = g_strdup (p);
-                        break;
-                }
-        }
-
-        freeifaddrs (ifa_list);
-
-        if (!ret) {
-                /* Didn't find anything. Let's take the loopback IP. */
-                ret = g_strdup (LOOPBACK_IP);
-        }
-
-        return ret;
-}
-
-/*
- * Get the host IP of the interface used for the default route.  On any error,
- * the first up and non-loopback interface is used.
- */
-static char *
-get_default_host_ip (void)
-{
-        FILE *fp;
-        int ret;
-        char dev[32];
-        unsigned long dest;
-        gboolean found = FALSE;
-        
-#if defined(__FreeBSD__)
-	if ((fp = popen ("netstat -r -f inet -n -W", "r"))) {
-		char buffer[BUFSIZ];
-
-		char destination[32];
-
-		int i;
-		/* Skip the 4 header lines */
-		for (i=0;i<4;i++) {
-			if (!(fgets(buffer, BUFSIZ, fp)))
-				return NULL; /* Can't read */
-
-			if (buffer[strlen(buffer)-1] != '\n') {
-				g_warning("Can't read netstat output!");
-				return NULL;
-			}
-		}
-
-		while (fgets(buffer, BUFSIZ, fp)) {
-			if (buffer[strlen(buffer)-1] != '\n') {
-				g_warning("Can't read netstat output!");
-				return NULL;
-			}
-
-			if (sscanf(buffer, "%s %*s %*s %*d %*d %*d %s %*d", destination, dev) == 2) {
-				if (strcmp("default", destination) == 0) {
-					found = TRUE;
-					break;
-				}
-			}
-		}
-		pclose(fp);
-	}
-#else
-        /* TODO: error checking */
-
-        fp = fopen ("/proc/net/route", "r");
-        
-        /* Skip the header */
-        if (fscanf (fp, "%*[^\n]\n") == EOF) {
-               fclose (fp);
-               return NULL;
-	}
-
-        while ((ret = fscanf (fp,
-                              "%31s %lx %*x %*X %*d %*d %*d %*x %*d %*d %*d",
-                              dev, &dest)) != EOF) {
-                /* If we got a device name and destination, and the destination
-                   is 0, then we have the default route */
-                if (ret == 2 && dest == 0) {
-                        found = TRUE;
-                        break;
-                }
-        }
-        
-        fclose (fp);
-#endif
-
-        return get_host_ip (found ? dev : NULL);
-}
-
 static void
 gupnp_context_init (GUPnPContext *context)
 {
@@ -299,9 +152,6 @@ gupnp_context_set_property (GObject      *object,
         context = GUPNP_CONTEXT (object);
 
         switch (property_id) {
-        case PROP_HOST_IP:
-                context->priv->host_ip = g_value_dup_string (value);
-                break;
         case PROP_PORT:
                 context->priv->port = g_value_get_uint (value);
                 break;
@@ -325,10 +175,6 @@ gupnp_context_get_property (GObject    *object,
         context = GUPNP_CONTEXT (object);
 
         switch (property_id) {
-        case PROP_HOST_IP:
-                g_value_set_string (value,
-                                    gupnp_context_get_host_ip (context));
-                break;
         case PROP_PORT:
                 g_value_set_uint (value,
                                   gupnp_context_get_port (context));
@@ -383,8 +229,6 @@ gupnp_context_finalize (GObject *object)
 
         context = GUPNP_CONTEXT (object);
 
-        g_free (context->priv->host_ip);
-
         g_free (context->priv->server_url);
 
         /* Call super */
@@ -407,24 +251,6 @@ gupnp_context_class_init (GUPnPContextClass *klass)
 
         g_type_class_add_private (klass, sizeof (GUPnPContextPrivate));
 
-        /**
-         * GUPnPContext:host-ip
-         *
-         * The local host's IP address. Set to NULL to autodetect.
-         **/
-        g_object_class_install_property
-                (object_class,
-                 PROP_HOST_IP,
-                 g_param_spec_string ("host-ip",
-                                      "Host IP",
-                                      "The local host's IP address",
-                                      NULL,
-                                      G_PARAM_READWRITE |
-                                      G_PARAM_CONSTRUCT_ONLY |
-                                      G_PARAM_STATIC_NAME |
-                                      G_PARAM_STATIC_NICK |
-                                      G_PARAM_STATIC_BLURB));
-
         /**
          * GUPnPContext:port
          *
@@ -571,15 +397,15 @@ make_server_url (GUPnPContext *context)
         SoupServer *server;
         guint port;
 
-        if (context->priv->host_ip == NULL)
-                context->priv->host_ip = get_default_host_ip ();
-
         /* What port are we running on? */
         server = gupnp_context_get_server (context);
         port = soup_server_get_port (server);
 
         /* Put it all together */
-        return g_strdup_printf ("http://%s:%u", context->priv->host_ip, port);
+        return g_strdup_printf
+                        ("http://%s:%u",
+                         gssdp_client_get_host_ip (GSSDP_CLIENT (context)),
+                         port);
 }
 
 const char *
@@ -594,24 +420,24 @@ _gupnp_context_get_server_url (GUPnPContext *context)
 /**
  * gupnp_context_new
  * @main_context: A #GMainContext, or %NULL to use the default one
- * @host_ip: The local host's IP address, or %NULL to use the IP address
- * of the first non-loopback network interface.
+ * @interface: The network interface to use, or %NULL to auto-detect.
  * @port: Port to run on, or 0 if you don't care what port is used.
  * @error: A location to store a #GError, or %NULL
  *
- * Create a new #GUPnPContext with the specified @main_context, @host_ip and @port.
+ * Create a new #GUPnPContext with the specified @main_context, @interface and
+ * @port.
  *
  * Return value: A new #GUPnPContext object, or %NULL on an error
  **/
 GUPnPContext *
 gupnp_context_new (GMainContext *main_context,
-                   const char   *host_ip,
+                   const char   *interface,
                    guint         port,
                    GError      **error)
 {
         return g_object_new (GUPNP_TYPE_CONTEXT,
                              "main-context", main_context,
-                             "host-ip", host_ip,
+                             "interface", interface,
                              "port", port,
                              "error", error,
                              NULL);
@@ -624,16 +450,15 @@ gupnp_context_new (GMainContext *main_context,
  * Get the IP address we advertise ourselves as using.
  *
  * Return value: The IP address. This string should not be freed.
+ *
+ * Deprecated:0.12.7: The "host-ip" property has moved to the base class
+ * #GSSDPClient so newer applications should use
+ * #gssdp_client_get_host_ip instead.
  **/
 const char *
 gupnp_context_get_host_ip (GUPnPContext *context)
 {
-        g_return_val_if_fail (GUPNP_IS_CONTEXT (context), NULL);
-
-        if (context->priv->host_ip == NULL)
-                context->priv->host_ip = get_default_host_ip ();
-
-        return context->priv->host_ip;
+        return gssdp_client_get_host_ip (GSSDP_CLIENT (context));
 }
 
 /**
@@ -923,8 +748,10 @@ host_path_handler (SoupServer        *server,
                 g_free (length);
 
         } else {
-                g_assert_not_reached ();
+                soup_message_set_status (msg,
+                                         SOUP_STATUS_METHOD_NOT_ALLOWED);
 
+                goto DONE;
         }
 
         /* Set Content-Type */
