@@ -24,6 +24,7 @@
 
 #include <libgssdp/gssdp-client.h>
 #include <libsoup/soup-server.h>
+#include <libsoup/soup-session.h>
 
 G_BEGIN_DECLS
 
@@ -77,19 +78,26 @@ typedef struct {
 
 GUPnPContext *
 gupnp_context_new                      (GMainContext *main_context,
-                                        const char   *host_ip,
+                                        const char   *interface,
                                         guint         port,
                                         GError      **error);
 
+#ifndef GUPNP_DISABLE_DEPRECATED
+
 const char *
 gupnp_context_get_host_ip              (GUPnPContext *context);
 
+#endif /* GUPNP_DISABLE_DEPRECATED */
+
 guint
 gupnp_context_get_port                 (GUPnPContext *context);
 
 SoupServer *
 gupnp_context_get_server               (GUPnPContext *context);
 
+SoupSession *
+gupnp_context_get_session              (GUPnPContext *context);
+
 void
 gupnp_context_set_subscription_timeout (GUPnPContext *context,
                                         guint         timeout);
