@@ -48,7 +48,7 @@ struct _GUPnPDeviceInfoPrivate {
 
         SoupURI *url_base;
 
-        XmlDocWrapper *doc;
+        GUPnPXMLDoc *doc;
 
         xmlNode *element;
 };
@@ -108,10 +108,7 @@ gupnp_device_info_set_property (GObject      *object,
 
                 break;
         case PROP_DOCUMENT:
-                info->priv->doc = g_value_get_object (value);
-                if (info->priv->doc)
-                        g_object_ref_sink (info->priv->doc);
-
+                info->priv->doc = g_value_dup_object (value);
                 break;
         case PROP_ELEMENT:
                 info->priv->element = g_value_get_pointer (value);
@@ -340,7 +337,7 @@ gupnp_device_info_class_init (GUPnPDeviceInfoClass *klass)
                                       "Document",
                                       "The XML document related to this "
                                       "device",
-                                      TYPE_XML_DOC_WRAPPER,
+                                      GUPNP_TYPE_XML_DOC,
                                       G_PARAM_WRITABLE |
                                       G_PARAM_CONSTRUCT_ONLY |
                                       G_PARAM_STATIC_NAME |
@@ -888,7 +885,7 @@ resource_type_match (const char *query,
 
         /* Inspect last colon (if any!) on @base */
         colon = strrchr (base, ':');
-        if (G_UNLIKELY (*colon == 0))
+        if (G_UNLIKELY (colon == NULL))
                 return !strcmp (query, base); /* No colon */
 
         /* Length of type until last colon */
@@ -927,6 +924,83 @@ resource_type_match (const char *query,
         return (query_ver <= base_ver);
 }
 
+/**
+ * gupnp_device_info_list_dlna_capabilities
+ * @info: A #GUPnPDeviceInfo
+ *
+ * Get a #GList of strings that represent the device capabilities as
+ * announced in the device description file using the
+ * &lt;dlna:X_DLNACAP&gt; element.
+ *
+ * Return value: a #GList of newly allocated strings or %NULL if
+ *               the device description doesn't contain the
+ *               &lt;dlna:X_DLNACAP&gt; element.
+ **/
+GList *
+gupnp_device_info_list_dlna_capabilities (GUPnPDeviceInfo *info)
+{
+        xmlChar *caps;
+
+        g_return_val_if_fail (GUPNP_IS_DEVICE_INFO (info), NULL);
+
+        caps = xml_util_get_child_element_content (info->priv->element,
+                                                   "X_DLNACAP");
+
+        if (caps) {
+                GList         *list  = NULL;
+                const xmlChar *start = caps;
+
+                while (*start) {
+                        const xmlChar *end = start;
+
+                        while (*end && *end != ',')
+                                end++;
+
+                        if (end > start) {
+                                gchar *value;
+
+                                value = g_strndup ((const gchar *) start,
+                                                   end - start);
+
+                                list = g_list_prepend (list, value);
+                        }
+
+                        if (*end)
+                                start = end + 1;
+                        else
+                                break;
+                }
+
+                xmlFree (caps);
+
+                return g_list_reverse (list);
+        }
+
+        return NULL;
+}
+
+/**
+ * gupnp_device_info_get_description_value
+ * @info:    A #GUPnPDeviceInfo
+ * @element: Name of the description element to retrieve
+ *
+ * This function provides generic access to the contents of arbitrary
+ * elements in the device description file.
+ *
+ * Return value: a newly allocated string or %NULL if the device
+ *               description doesn't contain the given @element
+ **/
+char *
+gupnp_device_info_get_description_value (GUPnPDeviceInfo *info,
+                                         const char      *element)
+{
+        g_return_val_if_fail (GUPNP_IS_DEVICE_INFO (info), NULL);
+        g_return_val_if_fail (element != NULL, NULL);
+
+        return xml_util_get_child_element_content_glib (info->priv->element,
+                                                        element);
+}
+
 /**
  * gupnp_device_info_list_devices
  * @info: A #GUPnPDeviceInfo
@@ -1239,8 +1313,8 @@ gupnp_device_info_get_service (GUPnPDeviceInfo *info,
         return service;
 }
 
-/* Return associated xmlDoc wrapper */
-XmlDocWrapper *
+/* Return associated xmlDoc */
+GUPnPXMLDoc *
 _gupnp_device_info_get_document (GUPnPDeviceInfo *info)
 {
         return info->priv->doc;
