@@ -27,6 +27,7 @@
 #include "gupnp-context.h"
 #include "gupnp-device.h"
 #include "gupnp-resource-factory.h"
+#include "gupnp-xml-doc.h"
 
 G_BEGIN_DECLS
 
@@ -80,13 +81,15 @@ typedef struct {
 
 GUPnPRootDevice *
 gupnp_root_device_new             (GUPnPContext         *context,
-                                   const char           *relative_location);
+                                   const char           *description_path,
+                                   const char           *description_dir);
 
 GUPnPRootDevice *
 gupnp_root_device_new_full        (GUPnPContext         *context,
                                    GUPnPResourceFactory *factory,
-                                   xmlDoc               *description_doc,
-                                   const char           *relative_location);
+                                   GUPnPXMLDoc          *description_doc,
+                                   const char           *description_path,
+                                   const char           *description_dir);
 
 void
 gupnp_root_device_set_available   (GUPnPRootDevice      *root_device,
@@ -99,6 +102,14 @@ const char *
 gupnp_root_device_get_relative_location
                                   (GUPnPRootDevice      *root_device);
 
+const char *
+gupnp_root_device_get_description_path
+                                  (GUPnPRootDevice      *root_device);
+
+const char *
+gupnp_root_device_get_description_dir
+                                  (GUPnPRootDevice      *root_device);
+
 G_END_DECLS
 
 #endif /* __GUPNP_ROOT_DEVICE_H__ */
