@@ -195,6 +195,7 @@ gupnp_service_info_dispose (GObject *object)
                                                      data->message,
                                                      SOUP_STATUS_CANCELLED);
 
+                        g_object_unref (data->message);
                         get_scpd_url_data_free (data);
 
                         info->priv->pending_gets =
@@ -543,6 +544,9 @@ gupnp_service_info_get_event_subscription_url (GUPnPServiceInfo *info)
  * description document (SCPD) provided by the service so it can not be created
  * if the service does not provide an SCPD.
  *
+ * Warning: You  should use gupnp_service_info_get_introspection_async()
+ * instead, this function re-enter the GMainloop before returning.
+ *
  * Return value: A new #GUPnPServiceIntrospection for this service or %NULL.
  * Unref after use.
  **/
