@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2007 Zeeshan Ali <zeenix@gstreamer.net>
+ * Copyright (C) 2007 Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  * Copyright (C) 2006, 2007 OpenedHand Ltd.
  *
- * Author: Zeeshan Ali <zeenix@gstreamer.net>
+ * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *         Jorn Baayen <jorn@openedhand.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -729,9 +729,10 @@ collect_variable_names (gpointer data,
  * gupnp_service_introspection_new
  * @scpd: Pointer to the SCPD of the service to create a introspection for
  *
- * Return value: A #GUPnPServiceIntrospection for the service created from the
- * SCPD @scpd or NULL.
+ * Create a new #GUPnPServiceIntrospection for the service created from the
+ * SCPD @scpd or %NULL.
  *
+ * Return value: A new #GUPnPServiceIntrospection.
  **/
 GUPnPServiceIntrospection *
 gupnp_service_introspection_new (xmlDoc *scpd)
@@ -759,7 +760,7 @@ gupnp_service_introspection_new (xmlDoc *scpd)
  *
  * Returns a GList of names of all the actions in this service.
  *
- * Return value: A GList of names of all the actions or NULL. Do not modify
+ * Return value: A GList of names of all the actions or %NULL. Do not modify
  * or free it or its contents.
  **/
 const GList *
@@ -782,10 +783,10 @@ gupnp_service_introspection_list_action_names
  * gupnp_service_introspection_list_actions
  * @introspection: A #GUPnPServiceIntrospection
  *
- * Returns a GList of all the actions (of type #GUPnPServiceActionInfo) in
+ * Returns a #GList of all the actions (of type #GUPnPServiceActionInfo) in
  * this service.
  *
- * Return value: A GList of all the actions or NULL. Do not modify or free it
+ * Return value: A #GList of all the actions or %NULL. Do not modify or free it
  * or its contents.
  *
  **/
@@ -803,7 +804,7 @@ gupnp_service_introspection_list_actions
  * Returns a GList of all the state variables (of type
  * #GUPnPServiceStateVariableInfo) in this service.
  *
- * Return value: A GList of all the state variables or NULL. Do not modify or
+ * Return value: A #GList of all the state variables or %NULL. Do not modify or
  * free it or its contents.
  *
  **/
@@ -818,9 +819,9 @@ gupnp_service_introspection_list_state_variables
  * gupnp_service_introspection_list_state_variable_names
  * @introspection: A #GUPnPServiceIntrospection
  *
- * Returns a GList of names of all the state variables in this service.
+ * Returns a #GList of names of all the state variables in this service.
  *
- * Return value: A GList of names of all the state variables or NULL. Do not
+ * Return value: A #GList of names of all the state variables or %NULL. Do not
  * modify or free it or its contents.
  **/
 const GList *
@@ -853,7 +854,7 @@ state_variable_search_func (GUPnPServiceStateVariableInfo *variable,
  *
  * Returns the state variable by the name @variable_name in this service.
  *
- * Return value: the state variable or NULL. Do not modify or free it.
+ * Return value: the state variable or %NULL. Do not modify or free it.
  **/
 const GUPnPServiceStateVariableInfo *
 gupnp_service_introspection_get_state_variable
@@ -869,6 +870,8 @@ gupnp_service_introspection_get_state_variable
                                 introspection->priv->variables,
                                 (gpointer) variable_name,
                                 (GCompareFunc) state_variable_search_func);
+        if (variable_node == NULL)
+                return NULL;
 
         return (GUPnPServiceStateVariableInfo *) variable_node->data;
 }
@@ -887,7 +890,7 @@ action_search_func (GUPnPServiceActionInfo *action,
  *
  * Returns the action by the name @action_name in this service.
  *
- * Return value: the action or NULL. Do not modify or free it.
+ * Return value: the action or %NULL. Do not modify or free it.
  **/
 const GUPnPServiceActionInfo *
 gupnp_service_introspection_get_action
@@ -903,6 +906,8 @@ gupnp_service_introspection_get_action
                                 introspection->priv->actions,
                                 (gpointer) action_name,
                                 (GCompareFunc) action_search_func);
+        if (action_node == NULL)
+                return NULL;
 
         return (GUPnPServiceActionInfo *) action_node->data;
 }
