@@ -106,6 +106,13 @@ typedef struct {
         gpointer user_data;
 } CallbackData;
 
+typedef struct {
+        char *sid;
+        int seq;
+
+        xmlDoc *doc;
+} EmitNotifyData;
+
 static void
 subscribe_got_response (SoupSession       *session,
                         SoupMessage       *msg,
@@ -135,6 +142,32 @@ notify_data_free (NotifyData *data)
         g_slice_free (NotifyData, data);
 }
 
+/* Steals doc reference */
+static EmitNotifyData *
+emit_notify_data_new (const char *sid,
+                      int         seq,
+                      xmlDoc     *doc)
+{
+        EmitNotifyData *data;
+
+        data = g_slice_new (EmitNotifyData);
+
+        data->sid = g_strdup (sid);
+        data->seq = seq;
+        data->doc = doc;
+
+        return data;
+}
+
+static void
+emit_notify_data_free (EmitNotifyData *data)
+{
+        g_free (data->sid);
+        xmlFreeDoc (data->doc);
+
+        g_slice_free (EmitNotifyData, data);
+}
+
 static void
 gupnp_service_proxy_init (GUPnPServiceProxy *proxy)
 {
@@ -252,7 +285,7 @@ gupnp_service_proxy_dispose (GObject *object)
         }
         
         while (proxy->priv->pending_notifies) {
-                xmlFreeDoc (proxy->priv->pending_notifies->data);
+                emit_notify_data_free (proxy->priv->pending_notifies->data);
                 proxy->priv->pending_notifies =
                         g_list_delete_link (proxy->priv->pending_notifies,
                                             proxy->priv->pending_notifies);
@@ -1325,79 +1358,117 @@ gupnp_service_proxy_remove_notify (GUPnPServiceProxy              *proxy,
         return found;
 }
 
+static void emit_notification (GUPnPServiceProxy *proxy,
+                               xmlDoc            *doc)
+{
+        xmlNode *node;
+
+        node = xmlDocGetRootElement (doc);
+
+        /* Iterate over all provided properties */
+        for (node = node->children; node; node = node->next) {
+                xmlNode *var_node;
+                NotifyData *data;
+                GValue value = {0, };
+                GList *l;
+
+                /* variableName node */
+                var_node = node->children;
+
+                if (var_node == NULL ||
+                    strcmp ((char *) node->name, "property") != 0)
+                        continue;
+
+                data = g_hash_table_lookup (proxy->priv->notify_hash,
+                                            var_node->name);
+                if (data == NULL)
+                        continue;
+
+                /* Make a GValue of the desired type */
+                g_value_init (&value, data->type);
+
+                if (!gvalue_util_set_value_from_xml_node (&value, var_node)) {
+                        g_value_unset (&value);
+
+                        continue;
+                }
+
+                /* Call callbacks */
+                for (l = data->callbacks; l; l = l->next) {
+                        CallbackData *callback_data;
+
+                        callback_data = l->data;
+
+                        callback_data->callback (proxy,
+                                                 (const char *) var_node->name,
+                                                 &value,
+                                                 callback_data->user_data);
+                }
+
+                /* Cleanup */
+                g_value_unset (&value);
+        }
+}
+
 /* Emit pending notifications. See comment below on why we do this. */
 static gboolean
 emit_notifications (gpointer user_data)
 {
         GUPnPServiceProxy *proxy = user_data;
+        GList *pending_notify;
+        gboolean resubscribe = FALSE;
 
         g_assert (user_data);
-        
-        while (proxy->priv->pending_notifies != NULL) {
-                xmlDoc *doc;
-                xmlNode *node;
 
-                doc = proxy->priv->pending_notifies->data;
-                node = xmlDocGetRootElement (doc);
-                
-                /* Iterate over all provided properties */
-                for (node = node->children; node; node = node->next) {
-                        xmlNode *var_node;
-                        
-                        if (strcmp ((char *) node->name, "property") != 0)
-                                continue;
-                        
-                        /* property */
-                        for (var_node = node->children; var_node;
-                             var_node = var_node->next) {
-                                NotifyData *data;
-                                GValue value = {0, };
-                                GList *l;
-                                
-                                data = g_hash_table_lookup
-                                        (proxy->priv->notify_hash,
-                                         var_node->name);
-                                if (data == NULL)
-                                        continue;
-                                
-                                /* Make a GValue of the desired type */
-                                g_value_init (&value, data->type);
-                                
-                                if (!gvalue_util_set_value_from_xml_node
-                                                        (&value, var_node)) {
-                                        g_value_unset (&value);
-                                        
-                                        continue;
-                                }
-                                
-                                /* Call callbacks */
-                                for (l = data->callbacks; l; l = l->next) {
-                                        CallbackData *callback_data;
-                                        
-                                        callback_data = l->data;
-                                        
-                                        callback_data->callback
-                                                (proxy,
-                                                 (const char *) var_node->name,
-                                                 &value,
-                                                 callback_data->user_data);
-                                }
-                                
-                                /* Cleanup */
-                                g_value_unset (&value);
-                        }
+        if (proxy->priv->sid == NULL)
+                /* No SID */
+                if (G_LIKELY (proxy->priv->subscribed))
+                        /* subscription in progress, delay emision! */
+                        return TRUE;
+
+        for (pending_notify = proxy->priv->pending_notifies;
+             pending_notify != NULL;
+             pending_notify = pending_notify->next) {
+                EmitNotifyData *emit_notify_data;
+
+                emit_notify_data = pending_notify->data;
+
+                if (emit_notify_data->seq > proxy->priv->seq) {
+                        /* Oops, we missed a notify. Resubscribe .. */
+                        resubscribe = TRUE;
+
+                        break;
                 }
-                
-                /* Cleanup */
-                xmlFreeDoc (doc);
+
+                /* Increment our own event sequence number */
+                if (proxy->priv->seq < G_MAXINT32)
+                        proxy->priv->seq++;
+                else
+                        proxy->priv->seq = 1;
+
+                if (G_LIKELY (proxy->priv->sid != NULL &&
+                              strcmp (emit_notify_data->sid,
+                                      proxy->priv->sid) == 0))
+                        /* Our SID, entertain! */
+                        emit_notification (proxy, emit_notify_data->doc);
+        }
+
+        /* Cleanup */
+        while (proxy->priv->pending_notifies != NULL) {
+                emit_notify_data_free (proxy->priv->pending_notifies->data);
 
                 proxy->priv->pending_notifies =
                         g_list_delete_link (proxy->priv->pending_notifies,
                                             proxy->priv->pending_notifies);
         }
-        
+
         proxy->priv->notify_idle_src = NULL;
 
+        if (resubscribe) {
+                unsubscribe (proxy);
+                subscribe (proxy);
+        }
+
 	return FALSE;
 }
 
@@ -1418,6 +1489,7 @@ server_handler (SoupServer        *soup_server,
         int seq;
         xmlDoc *doc;
         xmlNode *node;
+        EmitNotifyData *emit_notify_data;
 
         proxy = GUPNP_SERVICE_PROXY (user_data);
 
@@ -1444,41 +1516,6 @@ server_handler (SoupServer        *soup_server,
                 return;
         }
 
-        hdr = soup_message_headers_get (msg->request_headers, "SID");
-        if (hdr == NULL ||
-            (proxy->priv->sid && (strcmp (hdr, proxy->priv->sid) != 0))) {
-                /* No SID or not ours */
-                soup_message_set_status (msg, SOUP_STATUS_PRECONDITION_FAILED);
-
-                return;
-        }
-
-        /* We do not error out if proxy->priv->sid is NIL as the subscription
-         * response may not have been processed yet. */
-#if 0
-        if (!proxy->priv->sid) {
-                GUPnPContext *context;
-                GMainContext *main_context;
-
-                context = gupnp_service_info_get_context
-                                        (GUPNP_SERVICE_INFO (proxy));
-                main_context = gssdp_client_get_main_context
-                                        (GSSDP_CLIENT (context));
-
-                /* Wait. Perhaps the subscription response has not yet
-                 * been processed. */
-                g_main_context_iteration (main_context, FALSE);
-
-                if (!proxy->priv->sid || strcmp (hdr, proxy->priv->sid) != 0) {
-                        /* Really not our SID */
-                        soup_message_set_status
-                                (msg, SOUP_STATUS_PRECONDITION_FAILED);
-
-                        return;
-                }
-        }
-#endif
-
         hdr = soup_message_headers_get (msg->request_headers, "SEQ");
         if (hdr == NULL) {
                 /* No SEQ header */
@@ -1488,23 +1525,15 @@ server_handler (SoupServer        *soup_server,
         }
 
         seq = atoi (hdr);
-        if (seq > proxy->priv->seq) {
-                /* Oops, we missed a notify. Resubscribe .. */
-                unsubscribe (proxy);
-                subscribe (proxy);
 
-                /* Message was OK otherwise */
-                soup_message_set_status (msg, SOUP_STATUS_OK);
+        hdr = soup_message_headers_get (msg->request_headers, "SID");
+        if (hdr == NULL) {
+                /* No SID */
+                soup_message_set_status (msg, SOUP_STATUS_PRECONDITION_FAILED);
 
                 return;
         }
 
-        /* Increment our own event sequence number */
-        if (proxy->priv->seq < G_MAXINT32)
-                proxy->priv->seq++;
-        else
-                proxy->priv->seq = 1;
-
         /* Parse the actual XML message content */
         doc = xmlRecoverMemory (msg->request_body->data,
                                 msg->request_body->length);
@@ -1534,8 +1563,10 @@ server_handler (SoupServer        *soup_server,
 	 * call the callbacks in an idle handler so that if the client calls the
 	 * device in the notify callback the server can actually respond.
 	 */
+        emit_notify_data = emit_notify_data_new (hdr, seq, doc);
+
         proxy->priv->pending_notifies =
-                g_list_append (proxy->priv->pending_notifies, doc);
+                g_list_append (proxy->priv->pending_notifies, emit_notify_data);
         if (!proxy->priv->notify_idle_src) {
 	        GUPnPContext *context;
 	        GMainContext *main_context;
@@ -1670,7 +1701,7 @@ subscribe_got_response (SoupSession       *session,
                                          GUPNP_EVENTING_ERROR_SUBSCRIPTION_LOST,
                                          "No SID in SUBSCRIBE response");
 
-                        goto ERROR;
+                        goto hdr_err;
                 }
 
                 proxy->priv->sid = g_strdup (hdr);
@@ -1730,7 +1761,7 @@ subscribe_got_response (SoupSession       *session,
                                  GUPNP_EVENTING_ERROR_SUBSCRIPTION_FAILED,
                                  msg->reason_phrase);
 
- ERROR:
+hdr_err:
                 proxy->priv->subscribed = FALSE;
 
                 g_object_notify (G_OBJECT (proxy), "subscribed");
@@ -1855,11 +1886,15 @@ unsubscribe (GUPnPServiceProxy *proxy)
         SoupServer *server;
         char *sub_url;
 
+        context = gupnp_service_info_get_context (GUPNP_SERVICE_INFO (proxy));
+
+        /* Remove server handler */
+        server = gupnp_context_get_server (context);
+        soup_server_remove_handler (server, proxy->priv->path);
+
         if (proxy->priv->sid == NULL)
                 return; /* No SID: nothing to unsubscribe */
 
-        context = gupnp_service_info_get_context (GUPNP_SERVICE_INFO (proxy));
-
         /* Create unsubscription message */
         sub_url = gupnp_service_info_get_event_subscription_url
                                                 (GUPNP_SERVICE_INFO (proxy));
@@ -1892,10 +1927,6 @@ unsubscribe (GUPnPServiceProxy *proxy)
                 g_source_destroy (proxy->priv->subscription_timeout_src);
                 proxy->priv->subscription_timeout_src = NULL;
         }
-
-        /* Remove server handler */
-        server = gupnp_context_get_server (context);
-        soup_server_remove_handler (server, proxy->priv->path);
 }
 
 /**
