@@ -98,6 +98,9 @@ typedef struct {
                                            subscription */
 } SubscriptionData;
 
+static void
+send_initial_state (SubscriptionData *data);
+
 static SoupSession *
 gupnp_service_get_session (GUPnPService *service)
 {
@@ -586,6 +589,21 @@ gupnp_service_action_return_error (GUPnPServiceAction *action,
         finalize_action (action);
 }
 
+/**
+ * gupnp_service_action_get_message
+ * @action: A #GUPnPServiceAction
+ *
+ * Get the #SoupMessage associated with @action. Mainly intended for
+ * applications to be able to read HTTP headers received from clients.
+ *
+ * Return value: #SoupMessage associated with @action. Unref after using it.
+ **/
+SoupMessage *
+gupnp_service_action_get_message (GUPnPServiceAction *action)
+{
+        return g_object_ref (action->msg);
+}
+
 static void
 gupnp_service_init (GUPnPService *service)
 {
@@ -723,6 +741,11 @@ control_server_handler (SoupServer        *server,
         /* Get action name */
         soap_action = soup_message_headers_get (msg->request_headers,
                                                 "SOAPAction");
+        if (!soap_action) {
+                soup_message_set_status (msg, SOUP_STATUS_PRECONDITION_FAILED);
+                return;
+        }
+
         action_name = strchr (soap_action, '#');
         if (!action_name) {
                 soup_message_set_status (msg, SOUP_STATUS_PRECONDITION_FAILED);
@@ -736,7 +759,8 @@ control_server_handler (SoupServer        *server,
 
         /* This memory is libsoup-owned so we can do this */
         end = strrchr (action_name, '"');
-        *end = '\0';
+        if (end)
+                *end = '\0';
 
         /* Parse action_node */
         doc = xmlRecoverMemory (msg->request_body->data,
@@ -872,7 +896,7 @@ parse_and_limit_timeout (const char *timeout)
 
         timeout_seconds = 0;
 
-        if (strncmp (timeout, "Second-", strlen ("Second-")) == 0) {
+        if (timeout && strncmp (timeout, "Second-", strlen ("Second-")) == 0) {
                 /* We have a finite timeout */
                 timeout_seconds = CLAMP (atoi (timeout + strlen ("Second-")),
                                          GENA_MIN_TIMEOUT,
@@ -883,6 +907,48 @@ parse_and_limit_timeout (const char *timeout)
         return timeout_seconds;
 }
 
+static void
+send_initial_state (SubscriptionData *data)
+{
+        GQueue *queue;
+        char *mem;
+        GList *l;
+
+        /* Send initial event message */
+        queue = g_queue_new ();
+
+        for (l = data->service->priv->state_variables; l; l = l->next) {
+                NotifyData *ndata;
+
+                ndata = g_slice_new0 (NotifyData);
+
+                g_signal_emit (data->service,
+                               signals[QUERY_VARIABLE],
+                               g_quark_from_string (l->data),
+                               l->data,
+                               &ndata->value);
+
+                if (!G_IS_VALUE (&ndata->value)) {
+                        g_slice_free (NotifyData, ndata);
+
+                        continue;
+                }
+
+                ndata->variable = g_strdup (l->data);
+
+                g_queue_push_tail (queue, ndata);
+        }
+
+        mem = create_property_set (queue);
+        notify_subscriber (data->sid, data, mem);
+
+        /* Cleanup */
+        g_queue_free (queue);
+
+        g_free (mem);
+}
+
+
 /* Subscription request */
 static void
 subscribe (GUPnPService *service,
@@ -893,9 +959,6 @@ subscribe (GUPnPService *service,
         SubscriptionData *data;
         char *start, *end, *uri;
         int timeout_seconds;
-        GQueue *queue;
-        char *mem;
-        GList *l;
 
         data = g_slice_new0 (SubscriptionData);
 
@@ -960,38 +1023,7 @@ subscribe (GUPnPService *service,
         /* Respond */
         subscription_response (service, msg, data->sid, timeout_seconds);
 
-        /* Send initial event message */
-        queue = g_queue_new ();
-
-        for (l = service->priv->state_variables; l; l = l->next) {
-                NotifyData *ndata;
-
-                ndata = g_slice_new0 (NotifyData);
-
-                g_signal_emit (service,
-                               signals[QUERY_VARIABLE],
-                               g_quark_from_string (l->data),
-                               l->data,
-                               &ndata->value);
-
-                if (!G_IS_VALUE (&ndata->value)) {
-                        g_slice_free (NotifyData, ndata);
-
-                        continue;
-                }
-
-                ndata->variable = g_strdup (l->data);
-
-                g_queue_push_tail (queue, ndata);
-        }
-
-        mem = create_property_set (queue);
-        notify_subscriber (data->sid, data, mem);
-
-        /* Cleanup */
-        g_queue_free (queue);
-
-        g_free (mem);
+        send_initial_state (data);
 }
 
 /* Resubscription request */
@@ -1127,6 +1159,50 @@ subscription_server_handler (SoupServer        *server,
         }
 }
 
+void
+got_introspection (GUPnPServiceInfo *info,
+    GUPnPServiceIntrospection *introspection,
+    const GError *error,
+    gpointer user_data)
+{
+        GUPnPService *service = user_data;
+        const GList *state_variables, *l;
+        GHashTableIter iter;
+        SubscriptionData *data;
+
+        if (introspection) {
+                state_variables =
+                    gupnp_service_introspection_list_state_variables
+                    (introspection);
+
+                for (l = state_variables; l; l = l->next) {
+                        GUPnPServiceStateVariableInfo *variable;
+
+                        variable = l->data;
+
+                        if (!variable->send_events)
+                                continue;
+
+                        service->priv->state_variables =
+                            g_list_prepend (service->priv->state_variables,
+                                g_strdup (variable->name));
+                }
+
+                g_object_unref (introspection);
+        } else {
+                g_warning ("Failed to get SCPD: %s\n"
+                    "The initial event message will not be sent.",
+                    error ? error->message : "No error");
+        }
+
+        g_hash_table_iter_init (&iter, service->priv->subscriptions);
+
+        while (g_hash_table_iter_next (&iter, NULL, (gpointer*) &data))
+                send_initial_state (data);
+
+        g_object_unref (service);
+}
+
 static GObject *
 gupnp_service_constructor (GType                  type,
                            guint                  n_construct_params,
@@ -1136,9 +1212,6 @@ gupnp_service_constructor (GType                  type,
         GObject *object;
         GUPnPService *service;
         GUPnPServiceInfo *info;
-        GError *error;
-        GUPnPServiceIntrospection *introspection;
-        const GList *state_variables, *l;
         GUPnPContext *context;
         SoupServer *server;
         SoupURI *uri;
@@ -1155,35 +1228,10 @@ gupnp_service_constructor (GType                  type,
         info    = GUPNP_SERVICE_INFO (object);
 
         /* Get introspection and save state variable names */
-        error = NULL;
 
-        introspection = gupnp_service_info_get_introspection (info, &error);
-        if (introspection) {
-                state_variables =
-                        gupnp_service_introspection_list_state_variables
-                                                                (introspection);
-
-                for (l = state_variables; l; l = l->next) {
-                        GUPnPServiceStateVariableInfo *variable;
-
-                        variable = l->data;
-
-                        if (!variable->send_events)
-                                continue;
-
-                        service->priv->state_variables =
-                                g_list_prepend (service->priv->state_variables,
-                                                g_strdup (variable->name));
-                }
-
-                g_object_unref (introspection);
-        } else {
-                g_warning ("Failed to get SCPD: %s\n"
-                           "The initial event message will not be sent.",
-                           error->message);
-
-                g_error_free (error);
-        }
+        gupnp_service_info_get_introspection_async (info, got_introspection,
+            object);
+        g_object_ref (object);
 
         /* Get server */
         context = gupnp_service_info_get_context (info);
@@ -1255,9 +1303,11 @@ gupnp_service_set_property (GObject      *object,
         switch (property_id) {
         case PROP_ROOT_DEVICE:
                 service->priv->root_device = g_value_get_object (value);
+                GUPnPRootDevice **dev = &(service->priv->root_device);
+
                 g_object_add_weak_pointer
                         (G_OBJECT (service->priv->root_device),
-                         (gpointer *) &service->priv->root_device);
+                         (gpointer *) dev);
 
                 service->priv->notify_available_id =
                         g_signal_connect_object (service->priv->root_device,
@@ -1303,6 +1353,8 @@ gupnp_service_dispose (GObject *object)
         service = GUPNP_SERVICE (object);
 
         if (service->priv->root_device) {
+                GUPnPRootDevice **dev = &(service->priv->root_device);
+
                 if (g_signal_handler_is_connected
                         (service->priv->root_device,
                          service->priv->notify_available_id)) {
@@ -1313,7 +1365,7 @@ gupnp_service_dispose (GObject *object)
 
                 g_object_remove_weak_pointer
                         (G_OBJECT (service->priv->root_device),
-                         (gpointer *) &service->priv->root_device);
+                         (gpointer *) dev);
 
                 service->priv->root_device = NULL;
         }
